%define anolis_release 2
%bcond_with rust
%define upstreamversion %{version}

Summary: A fast, lightweight Source Control Management system
Name: mercurial
Version: 6.3.3
Release: %{anolis_release}%{?dist}
License: GPLv2+
URL: https://mercurial-scm.org/
Source0: https://www.mercurial-scm.org/release/%{name}-%{upstreamversion}.tar.gz
Source1: mercurial-site-start.el
Patch0:  mercurial-rust-metadata.patch
BuildRequires: make
BuildRequires: bash-completion
BuildRequires: emacs-el
BuildRequires: emacs-nox
BuildRequires: gcc
BuildRequires: gettext
BuildRequires: pkgconfig
BuildRequires: python3-devel
BuildRequires: python3-setuptools
BuildRequires: python3-docutils
%if %{with rust}
BuildRequires: rust-packaging
%endif

Provides: hg = %{version}-%{release}
Requires: python3
Requires: emacs-filesystem
Recommends: python3-fb-re2

%description
Mercurial is a fast, lightweight source control management system designed
for efficient handling of very large distributed projects.

Quick start: https://www.mercurial-scm.org/wiki/QuickStart
Tutorial: https://www.mercurial-scm.org/wiki/Tutorial
Extensions: https://www.mercurial-scm.org/wiki/UsingExtensions


%package hgk
Summary:    Hgk interface for mercurial
Requires:   hg = %{version}-%{release}
Requires:   tk

%description hgk
A Mercurial extension for displaying the change history graphically
using Tcl/Tk.  Displays branches and merges in an easily
understandable way and shows diffs for each revision.  Based on
gitk for the git SCM.

Adds the "hg view" command.  See
https://www.mercurial-scm.org/wiki/HgkExtension for more
documentation.


%package chg
Summary:    A fast Mercurial command without slow Python startup
Requires:   hg = %{version}-%{release}

%description chg
chg is a C wrapper for the hg command. Typically, when you type hg, a new
Python process is created, Mercurial is loaded, and your requested command runs
and the process exits.

With chg, a Mercurial command server background process is created that runs
Mercurial. When you type chg, a C program connects to that background process
and executes Mercurial commands.

%if %{with rust}
%package rust
Summary:    Mercurial Rust binaries and extensions
# Effective license for the rust binaries, computed from statically linked dependencies:
# BSD
# GPLv2+
# MIT
# MIT or ASL 2.0
# MPLv2.0
# Python
# Unlicense or MIT
# zlib or ASL 2.0 or MIT
License:    GPLv2+
Requires:   hg = %{version}-%{release}

%description rust
This subpackage provides following Mercurial components implemented in Rust:

The `rustext` extension speeds up some functionality of Mercurial, e.g.
ancestry computations in revision graphs, status or discovery of differences
between repositories.

The experimental `rhg` executable implements a subset of the functionality of
`hg` using only Rust, to avoid the startup cost of a Python interpreter. This
subset is initially small but grows over time as `rhg` is improved. When
fallback to the Python implementation is configured, `rhg` aims to be a drop-in
replacement for `hg` that should behave the same, except that some commands run
faster.
%endif

%package doc
Summary: Documentation files for %{name}
Requires: %{name} = %{version}-%{release}
BuildArch: noarch

%description doc
The %{name}-doc package contains documentation files for %{name}.

%prep
%autosetup -p1 -n %{name}-%{upstreamversion}

%if %{with rust}
pushd rust
%cargo_prep
popd

%generate_buildrequires
for crate in rust/hg-core rust/hg-cpython rust/rhg; do
  cd $crate
  # Temporarily remove  hg-core = { path = "../hg-core"}  dependencies while generating buildrequires.
  sed -i.br -r -e '/=\s*\{[^}]+path\s*=/d' Cargo.toml
  %cargo_generate_buildrequires
  mv -f Cargo.toml{.br,}
  cd - >/dev/null
done
%endif
# /with rust

# These are shipped as examples in /usr/share/docs and should not be executable
chmod -x hgweb.cgi contrib/hgweb.fcgi

%build
PYTHON=%{python3} make all

# chg will invoke the 'hg' command - no direct Python dependency
pushd contrib/chg
make
popd

%if %{with rust}
pushd rust
%cargo_build
popd
%endif

%install
%{python3} setup.py install -O1 --root %{buildroot} --prefix %{_prefix} --record=%{name}.files
make install-doc DESTDIR=%{buildroot} MANDIR=%{_mandir}

# Overrule setup.py policy "c" for module usage: always allow rust extension (if available)
echo 'modulepolicy = b"rust+c-allow"' > %{buildroot}%{python3_sitearch}/mercurial/__modulepolicy__.py

%if %{with rust}
# We are not using the Mercurial build system to build rust, and must thus manually install relevant parts.
install -D -m 755 -pv rust/target/release/rhg %{buildroot}%{_bindir}
install -D -m 755 -pv rust/target/release/librusthg.so \
        %{buildroot}%{python3_sitearch}/mercurial/rustext%{python3_ext_suffix}
%endif

grep -v -e 'hgk.py*' \
        -e "%{python3_sitearch}/mercurial/" \
        -e "%{python3_sitearch}/hgext/" \
        -e "%{python3_sitearch}/hgext3rd/" \
        -e "%{python3_sitearch}/hgdemandimport/" \
        -e "%{_bindir}" \
        < %{name}.files > %{name}-base.files
grep 'hgk.py*' < %{name}.files > %{name}-hgk.files

install -D -m 755 contrib/hgk       %{buildroot}%{_libexecdir}/mercurial/hgk
install -m 755 contrib/hg-ssh       %{buildroot}%{_bindir}

bash_completion_dir=%{buildroot}$(pkg-config --variable=completionsdir bash-completion)
mkdir -p $bash_completion_dir
install -m 644 contrib/bash_completion $bash_completion_dir/hg

zsh_completion_dir=%{buildroot}%{_datadir}/zsh/site-functions
mkdir -p $zsh_completion_dir
install -m 644 contrib/zsh_completion $zsh_completion_dir/_mercurial

mkdir -p %{buildroot}%{_emacs_sitelispdir}/mercurial

pushd contrib
for file in mercurial.el mq.el; do
  #emacs -batch -l mercurial.el --no-site-file -f batch-byte-compile $file
  %{_emacs_bytecompile} $file
  install -p -m 644 $file ${file}c %{buildroot}%{_emacs_sitelispdir}/mercurial
  rm ${file}c
done
popd

pushd contrib/chg
make install DESTDIR=%{buildroot} PREFIX=%{_usr} MANDIR=%{_mandir}/man1
popd


mkdir -p %{buildroot}%{_sysconfdir}/mercurial/hgrc.d

mkdir -p %{buildroot}%{_emacs_sitestartdir} && install -m644 %SOURCE1 %{buildroot}%{_emacs_sitestartdir}

cat >hgk.rc <<EOF
[extensions]
# enable hgk extension ('hg help' shows 'view' as a command)
hgk=

[hgk]
path=%{_libexecdir}/mercurial/hgk
EOF
install -m 644 hgk.rc %{buildroot}%{_sysconfdir}/mercurial/hgrc.d

cat > certs.rc <<EOF
# see: https://www.mercurial-scm.org/wiki/CACertificates
[web]
cacerts = /etc/pki/tls/certs/ca-bundle.crt
EOF
install -m 644 certs.rc %{buildroot}%{_sysconfdir}/mercurial/hgrc.d

mv %{buildroot}%{python3_sitearch}/mercurial/locale %{buildroot}%{_datadir}/locale
rm -rf %{buildroot}%{python3_sitearch}/mercurial/locale

%find_lang hg

%generate_compatibility_deps

%files -f %{name}-base.files -f hg.lang
%license COPYING
%doc %attr(644,root,root) %{_mandir}/man?/hg*.zst
%doc %attr(644,root,root) contrib/*.svg
%dir %{python3_sitearch}/%{name}-*-py*.egg-info
%dir %{_datadir}/zsh/
%dir %{_datadir}/zsh/site-functions/
%dir %{_sysconfdir}/mercurial
%dir %{_sysconfdir}/mercurial/hgrc.d
%dir %{_datadir}/bash-completion/
%{_datadir}/zsh/site-functions/_mercurial
%if %{with rust}
%exclude %{python3_sitearch}/mercurial/rustext%{python3_ext_suffix}
%endif
%{python3_sitearch}/mercurial/
%{python3_sitearch}/hgext/
%{python3_sitearch}/hgext3rd/
%{python3_sitearch}/hgdemandimport/
%{_emacs_sitelispdir}/mercurial
%{_emacs_sitestartdir}/*.el
%{_bindir}/hg
%{_bindir}/hg-ssh
%config(noreplace) %{_sysconfdir}/mercurial/hgrc.d/certs.rc
%dir %{abidir}
%{abidir}/hg-option.list
%{abidir}/*.cpython-*-linux-gnu.dump

%files hgk -f %{name}-hgk.files
%{_libexecdir}/mercurial/
%config(noreplace) %{_sysconfdir}/mercurial/hgrc.d/hgk.rc

%files chg
%{_bindir}/chg
%doc %attr(644,root,root) %{_mandir}/man?/chg.*.zst

%if %{with rust}
%files rust
%{_bindir}/rhg
%{python3_sitearch}/mercurial/rustext%{python3_ext_suffix}
%endif

%files doc
%doc CONTRIBUTORS doc/README doc/hg*.html hgweb.cgi contrib/hgweb.fcgi contrib/hgweb.wsgi

%changelog
* Tue Mar 12 2024 Bo Ren <rb01097748@alibaba-inc.com> - 6.3.3-2
- Rebuild with python3.11

* Wed Mar 29 2023 happy_orange <songnannan@linux.alibaba.com> - 6.3.3-1
- init package
